"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _transformer = require("./transformer");

var _htmlparser = _interopRequireDefault(require("htmlparser2"));

var _defaultKey = require("./defaultKey");

var _lodash = require("lodash");

var _parserHTML = require("./parserHTML");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var PAIR_REG = /^\{\{(.*)}\}$/;
var NODE_TYPE = {
  ELEMENT: 'tag',
  TEXT: 'text',
  COMMENT: 'comment',
  SCRIPT: 'script',
  STYLE: 'style'
};

var HTMLtoJSX =
/*#__PURE__*/
function () {
  function HTMLtoJSX(config) {
    _classCallCheck(this, HTMLtoJSX);

    this.config = config || {};
    this.createClass = false;

    if (!this.config.indent) {
      this.config.indent = '  ';
    }
  }

  _createClass(HTMLtoJSX, [{
    key: "reset",
    value: function reset() {
      this.output = '';
      this.outputImportText = '';
      this.level = 0;
      this.scope = {};
    }
  }, {
    key: "convert",
    value: function convert(html) {
      this.reset();
      html = this._cleanInput(html);
      var nodes = (0, _parserHTML.getDomObject)(html);

      if (!this._onlyOneTopLevel(nodes)) {
        this.level++;
        html = "<div>\n".concat(html, "\n</div>");
        nodes = (0, _parserHTML.getDomObject)(html);
      }

      this._traverse({
        children: nodes
      });

      this.output = this.output.trim() + '\n';
      return {
        output: this.output,
        outputImportText: this.outputImportText
      };
    }
  }, {
    key: "_traverse",
    value: function _traverse(node) {
      var _this = this;

      node.children = node.children || [];
      this.level++;
      node.children.forEach(function (child) {
        _this._visit(child);
      });
      this.level--;
    }
  }, {
    key: "_visit",
    value: function _visit(node) {
      // remove enter
      if (node.data && /^\n(\s?)+$/.test(node.data)) {
        return;
      }

      node.attributes = [];

      for (var key in node.attribs) {
        node.attributes.push({
          name: key,
          value: node.attribs[key]
        });
      }

      this._beginVisit(node);

      this._traverse(node);

      this._endVisit(node);
    }
  }, {
    key: "_beginVisit",
    value: function _beginVisit(node) {
      this.output += '\n' + new Array(this.level - 1).join(' ');

      switch (node.type) {
        case NODE_TYPE.ELEMENT:
          this._beginVisitElement(node);

          break;

        case NODE_TYPE.TEXT:
          this._visitText(node);

          break;

        case NODE_TYPE.COMMENT:
          this._visitComment(node);

          break;

        case NODE_TYPE.SCRIPT:
          break;

        case NODE_TYPE.STYLE:
          break;

        default:
          console.warn('Unrecognised node type: ' + node.type);
      }
    }
  }, {
    key: "_endVisit",
    value: function _endVisit(node) {
      switch (node.type) {
        case NODE_TYPE.ELEMENT:
          this.output += '\n' + new Array(this.level + 1).join(' ');

          this._endVisitElement(node);

          break;

        case NODE_TYPE.TEXT:
        case NODE_TYPE.COMMENT:
        case NODE_TYPE.SCRIPT:
        case NODE_TYPE.STYLE:
          break;
      }
    }
  }, {
    key: "_beginVisitElement",
    value: function _beginVisitElement(node) {
      var _this2 = this;

      var tagName = node.name;
      var outputTagName = tagName;
      var attributes = [];
      node.attributes.forEach(function (attribute) {
        attributes.push(_this2._getElementAttribute(node, attribute));
      });
      this.output += (0, _transformer.transformFor)(node.attributes, true, this.scope);
      this.output += (0, _transformer.transformIf)(node.attributes, true, this.scope);
      this.output += "<".concat(outputTagName);

      if (attributes.length > 0) {
        this.output += ' ' + attributes.join(' ');
      }

      this.output += '>';
    }
  }, {
    key: "_endVisitElement",
    value: function _endVisitElement(node) {
      var tagName = node.name;
      var outputTagName = tagName;
      this.output = (0, _lodash.trimEnd)(this.output, this.config.indent);
      this.output += '</' + outputTagName + '>';
      this.output += (0, _transformer.transformFor)(node.attributes, false, this.scope);
      this.output += (0, _transformer.transformIf)(node.attributes, false, this.scope);
    }
  }, {
    key: "_visitText",
    value: function _visitText(node) {
      var _this3 = this;

      var text = node.data;

      if (PAIR_REG.test(text)) {
        text = text.replace(PAIR_REG, function (word, $1) {
          if (/^\{\{([props.].*)}\}$/.test(text)) {
            return "{".concat($1, "}");
          }

          return "{".concat((0, _transformer.transformPair)($1, _this3.scope), "}");
        });
      }

      this.output += text;
    }
  }, {
    key: "_visitComment",
    value: function _visitComment(node) {
      this.output += "{/*".concat(node.data.replace('*/', '* /'), "*/}");
    }
  }, {
    key: "_onlyOneTopLevel",
    value: function _onlyOneTopLevel(nodes) {
      var _rootNodes = nodes.filter(function (node) {
        return !/\n+/.test('\n');
      });

      return _rootNodes.length === 1;
    }
  }, {
    key: "_getElementAttribute",
    value: function _getElementAttribute(node, attribute) {
      switch (attribute.name) {
        case 'src':
          return "source={{uri: \"".concat(attribute.value, "\"}}");

        case 'class':
          var value = attribute.value.trim();
          var multiClass = value.split(' ');
          var style = '';

          if (multiClass.length === 1) {
            style = "styles.".concat(multiClass[0]);
          } else {
            style += '[';
            multiClass = multiClass.map(function (className) {
              return "styles.".concat(className);
            });
            style += multiClass.join(', ');
            style += ']';
          }

          return "style={".concat(style, "}");

        case _defaultKey.IF_KEY:
        case _defaultKey.FOR_KEY:
          break;

        default:
          var tagName = node.name;
          var name = attribute.name;
          var result = name; // Numeric values should be output as {123} not "123"

          if ((0, _lodash.isNumber)(attribute.value)) {
            result += "={".concat(attribute.value, "}");
          } else if (attribute.value.length > 0) {
            result += "=\"".concat(attribute.value.replace(/"/gm, '&quot;'), "\"");
          }

          return result;
      }
    }
  }, {
    key: "_cleanInput",
    value: function _cleanInput(html) {
      html = html.trim();
      return html;
    }
  }]);

  return HTMLtoJSX;
}();

exports.default = HTMLtoJSX;
module.exports = exports["default"];