"use strict";

var _loaderUtils = _interopRequireDefault(require("loader-utils"));

var _parserHTML = _interopRequireDefault(require("./parserHTML"));

var _package = _interopRequireDefault(require("../package.json"));

var _path = _interopRequireDefault(require("path"));

var _babelCore = require("babel-core");

var _getBabelConfig = _interopRequireDefault(require("./getBabelConfig"));

var _uppercamelcase = _interopRequireDefault(require("uppercamelcase"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = function (source) {
  this.cacheable();
  var context = this;
  var filePath = this.resourcePath;

  var fileBaseName = _path.default.basename(filePath, _path.default.extname(filePath));

  var query = _loaderUtils.default.parseQuery(this.query);

  var parseObject = (0, _parserHTML.default)(source);
  var output = '\nlet styles = {};\n';
  var componentName = (0, _uppercamelcase.default)(fileBaseName); // parse template

  var template = parseObject.template;

  if (template) {
    output += "let template = ".concat(getCodeString('template', template, filePath), ";\n");
  } // parse script


  var script = parseObject.script;

  if (script) {
    output += "let ".concat(componentName, " = ").concat(getCodeString('script', script, filePath), ";\n");
    output += "".concat(componentName, " = ").concat(componentName, ".__esModule ? ").concat(componentName, ".default : ").concat(componentName, ";\n");
  } // parse link stylesheet


  var styleSheetLinks = parseObject.styleSheetLinks;

  if (styleSheetLinks.length) {
    styleSheetLinks.forEach(function (link) {
      output += "styles = Object.assign(styles, require('!!stylesheet-loader!".concat(link, "'));\n");
    });
  } // parse style


  var styles = parseObject.styles;

  if (styles) {
    output += "styles = Object.assign(styles, ".concat(getCodeString('style', styles[0], filePath), ");\n");
  }

  function getCodeString(type, data, path) {
    var loaderString = '';

    switch (type) {
      case 'template':
        loaderString = "".concat(_package.default.name, "/lib/template-loader?").concat(JSON.stringify(query), "!").concat(_package.default.name, "/lib/node-loader?type=template&index=0!");
        break;

      case 'style':
        loaderString = "stylesheet-loader!".concat(_package.default.name, "/lib/node-loader?type=styles&index=0!");
        break;

      case 'script':
        loaderString = "babel-loader?".concat(JSON.stringify((0, _getBabelConfig.default)(query)), "!").concat(_package.default.name, "/lib/node-loader?").concat(JSON.stringify({
          type: 'script',
          index: 0,
          banner: query.banner
        }), "!");
        break;
    }

    return 'require(' + _loaderUtils.default.stringifyRequest(context, "!!".concat(loaderString).concat(path)) + ')';
  }

  output += "\nif (".concat(componentName, " && typeof ").concat(componentName, " === 'function') {\n  ").concat(componentName, ".prototype.render = function() {\n    return template.call(this, styles);\n  }\n}\n\nmodule.exports = ").concat(componentName, ";\n  ");
  return output;
};