"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformPair = transformPair;
exports.default = exports.hasForKey = exports.hasIfKey = exports.transformIf = exports.transformFor = void 0;

var babylon = _interopRequireWildcard(require("babylon"));

var _babelGenerator = _interopRequireDefault(require("babel-generator"));

var _babelTraverse = _interopRequireDefault(require("babel-traverse"));

var _defaultKey = require("./defaultKey");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

var FULL_VALUE_REG = /\{\{(.*)\}\}/g;

var transformFor = function transformFor(attributes) {
  var begin = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var scope = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var output = '';
  hasForKey(attributes, function (attribute) {
    var value = attribute.value.replace(FULL_VALUE_REG, '$1');
    value = value.split(' in '); // fall short of rule eg. '{{item in items}}'

    if (!value[0] || !value[1]) {
      return '';
    }

    if (begin) {
      scope[value[0]] = 1;
      output += "{props.".concat(value[1], ".map((").concat(value[0], ") => {return (");
    } else {
      scope[value[0]] = null;
      delete scope[value[0]];
      output = ');})}';
    }
  });
  return output;
}; // transform if block


exports.transformFor = transformFor;

var transformIf = function transformIf(attributes) {
  var begin = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var scope = arguments.length > 2 ? arguments[2] : undefined;
  var output = '';
  hasIfKey(attributes, function (attribute) {
    if (begin) {
      output += "{(".concat(attribute.value.replace(FULL_VALUE_REG, function (word, $1) {
        return transformPair($1, scope);
      }), ") && ");
    } else {
      output = '}';
    }
  });
  return output;
};

exports.transformIf = transformIf;

var hasIfKey = function hasIfKey(attributes, callback) {
  return hasKey(_defaultKey.IF_KEY, attributes, callback);
};

exports.hasIfKey = hasIfKey;

var hasForKey = function hasForKey(attributes, callback) {
  return hasKey(_defaultKey.FOR_KEY, attributes, callback);
};

exports.hasForKey = hasForKey;

var hasKey = function hasKey(keyName) {
  var attributes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var callback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function () {};
  var hasKey = false;
  attributes = Array.from(attributes);
  attributes.forEach(function (attribute) {
    if (attribute.name === keyName) {
      hasKey = true;
      callback(attribute);
    }
  });
  return hasKey;
};

function transformPair(code, scope) {
  var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var visitor = {
    noScope: 1,
    enter: function enter(path) {
      var node = path.node,
          parent = path.parent;

      if (node.type !== 'Identifier') {
        return;
      }

      var type = parent && parent.type;

      if ((type !== 'MemberExpression' || parent.object === node || parent.property === node && parent.computed) && (type !== 'ObjectProperty' || parent.key !== node) && !findScope(scope, node.name)) {
        node.name = "props.".concat(node.name);
      }
    }
  };
  var codeStr = code;
  var ast = babylon.parse("(".concat(codeStr, ")"));
  (0, _babelTraverse.default)(ast, visitor);
  var newCode = (0, _babelGenerator.default)(ast).code;

  if (newCode.charAt(newCode.length - 1) === ';') {
    newCode = newCode.slice(0, -1);
  }

  return "(".concat(newCode, ")");
}

function findScope(scope, name) {
  return scope[name];
}

exports["default"] = module.exports;
var _default = module.exports;
exports.default = _default;